@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -45,10 +45,12 @@ class CControllerProxyEdit extends CController {
 				(HOST_ENCRYPTION_NONE | HOST_ENCRYPTION_CERTIFICATE).','.
 				(HOST_ENCRYPTION_PSK | HOST_ENCRYPTION_CERTIFICATE).','.
 				(HOST_ENCRYPTION_NONE | HOST_ENCRYPTION_PSK | HOST_ENCRYPTION_CERTIFICATE),
-			'tls_issuer' => 	'db       hosts.tls_issuer',
 			'tls_psk' =>		'db       hosts.tls_psk',
 			'tls_psk_identity'=>'db       hosts.tls_psk_identity',
+			'psk_edit_mode' =>	'in 0,1',
+			'tls_issuer' => 	'db       hosts.tls_issuer',
 			'tls_subject' => 	'db       hosts.tls_subject',
+			'clone_proxyid' =>	'db       hosts.hostid',
 			'form_refresh' =>	'int32'
 		];
 
@@ -66,7 +68,7 @@ class CControllerProxyEdit extends CController {
 			return false;
 		}
 
-		if ($this->hasInput('proxyid')) {
+		if ($this->getInput('proxyid', 0)) {
 			return (bool) API::Proxy()->get([
 				'output' => [],
 				'proxyids' => $this->getInput('proxyid'),
@@ -92,20 +94,21 @@ class CControllerProxyEdit extends CController {
 			'description' => '',
 			'tls_accept' => HOST_ENCRYPTION_NONE,
 			'tls_connect' => HOST_ENCRYPTION_NONE,
-			'tls_issuer' => '',
 			'tls_psk' => '',
 			'tls_psk_identity' => '',
+			'psk_edit_mode' => 1,
+			'tls_issuer' => '',
 			'tls_subject' => '',
 			'form_refresh' => 0
 		];
 
 		// get values from the dabatase
-		if ($this->hasInput('proxyid')) {
+		if ($this->getInput('proxyid', 0)) {
 			$data['proxyid'] = $this->getInput('proxyid');
 
 			$proxies = API::Proxy()->get([
 				'output' => ['host', 'status', 'proxy_address', 'description', 'tls_connect', 'tls_accept',
-					'tls_issuer', 'tls_subject', 'tls_psk_identity', 'tls_psk'
+					'tls_issuer', 'tls_subject'
 				],
 				'selectInterface' => ['interfaceid', 'dns', 'ip', 'useip', 'port'],
 				'proxyids' => $data['proxyid']
@@ -115,12 +118,12 @@ class CControllerProxyEdit extends CController {
 			$data['host'] = $proxy['host'];
 			$data['status'] = $proxy['status'];
 			$data['proxy_address'] = $proxy['proxy_address'];
+			$data['description'] = $proxy['description'];
 			$data['tls_accept'] = $proxy['tls_accept'];
 			$data['tls_connect'] = $proxy['tls_connect'];
 			$data['tls_issuer'] = $proxy['tls_issuer'];
-			$data['tls_psk'] = $proxy['tls_psk'];
-			$data['tls_psk_identity'] = $proxy['tls_psk_identity'];
 			$data['tls_subject'] = $proxy['tls_subject'];
+
 			if ($data['status'] == HOST_STATUS_PROXY_PASSIVE) {
 				$data['interfaceid'] = $proxy['interface']['interfaceid'];
 				$data['dns'] = $proxy['interface']['dns'];
@@ -128,7 +131,10 @@ class CControllerProxyEdit extends CController {
 				$data['useip'] = $proxy['interface']['useip'];
 				$data['port'] = $proxy['interface']['port'];
 			}
-			$data['description'] = $proxy['description'];
+
+			if ($proxy['tls_connect'] == HOST_ENCRYPTION_PSK || ($proxy['tls_accept'] & HOST_ENCRYPTION_PSK)) {
+				$data['psk_edit_mode'] = 0;
+			}
 		}
 
 		// overwrite with input variables
@@ -142,9 +148,10 @@ class CControllerProxyEdit extends CController {
 		$data['description'] = $this->getInput('description', $data['description']);
 		$data['tls_accept'] = $this->getInput('tls_accept', $data['tls_accept']);
 		$data['tls_connect'] = $this->getInput('tls_connect', $data['tls_connect']);
+		$data['tls_psk_identity'] = $this->getInput('tls_psk_identity', '');
+		$data['tls_psk'] = $this->getInput('tls_psk', '');
+		$data['psk_edit_mode'] = $this->getInput('psk_edit_mode', $data['psk_edit_mode']);
 		$data['tls_issuer'] = $this->getInput('tls_issuer', $data['tls_issuer']);
-		$data['tls_psk'] = $this->getInput('tls_psk', $data['tls_psk']);
-		$data['tls_psk_identity'] = $this->getInput('tls_psk_identity', $data['tls_psk_identity']);
 		$data['tls_subject'] = $this->getInput('tls_subject', $data['tls_subject']);
 		$data['form_refresh'] = $this->getInput('form_refresh', $data['form_refresh']);
 
@@ -152,6 +159,10 @@ class CControllerProxyEdit extends CController {
 			$data['interfaceid'] = $this->getInput('interfaceid');
 		}
 
+		if (!$data['proxyid'] && $this->hasInput('clone_proxyid')) {
+			$data['clone_proxyid'] = $this->getInput('clone_proxyid');
+		}
+
 		$response = new CControllerResponseData($data);
 		$response->setTitle(_('Configuration of proxies'));
 		$this->setResponse($response);
