@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ class CControllerUserUnblock extends CController {
 	}
 
 	protected function checkPermissions() {
-		return ($this->getUserType() == USER_TYPE_SUPER_ADMIN);
+		return $this->checkAccess(CRoleHelper::UI_ADMINISTRATION_USERS);
 	}
 
 	protected function doAction() {
@@ -45,7 +45,7 @@ class CControllerUserUnblock extends CController {
 		DBstart();
 
 		$users = API::User()->get([
-			'output' => ['alias', 'name', 'surname'],
+			'output' => ['username', 'name', 'surname'],
 			'userids' => $userids,
 			'editable' => true
 		]);
@@ -54,9 +54,10 @@ class CControllerUserUnblock extends CController {
 
 		if ($result) {
 			foreach ($users as $user) {
-				info('User '.$user['alias'].' unblocked');
+				info('User '.$user['username'].' unblocked');
 				add_audit(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_USER,
-					'Unblocked user alias ['.$user['alias'].'] name ['.$user['name'].'] surname ['.$user['surname'].']'
+					'Unblocked user username ['.$user['username'].'] name ['.$user['name'].'] surname ['.
+					$user['surname'].']'
 				);
 			}
 		}
@@ -65,17 +66,17 @@ class CControllerUserUnblock extends CController {
 
 		$unblocked = count($userids);
 
-		$url = (new CUrl('zabbix.php'))
+		$response = new CControllerResponseRedirect((new CUrl('zabbix.php'))
 			->setArgument('action', 'user.list')
-			->setArgument('uncheck', '1');
-
-		$response = new CControllerResponseRedirect($url->getUrl());
+			->setArgument('page', CPagerHelper::loadPage('user.list', null))
+		);
 
 		if ($result) {
-			$response->setMessageOk(_n('User unblocked', 'Users unblocked', $unblocked));
+			$response->setFormData(['uncheck' => '1']);
+			CMessageHelper::setSuccessTitle(_n('User unblocked', 'Users unblocked', $unblocked));
 		}
 		else {
-			$response->setMessageError(_n('Cannot unblock user', 'Cannot unblock users', $unblocked));
+			CMessageHelper::setErrorTitle(_n('Cannot unblock user', 'Cannot unblock users', $unblocked));
 		}
 
 		$this->setResponse($response);
