@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -68,6 +68,12 @@ abstract class CControllerWidget extends CController {
 		return $this;
 	}
 
+	protected function getContext(): string {
+		return $this->hasInput('templateid')
+			? CWidgetConfig::CONTEXT_TEMPLATE_DASHBOARD
+			: CWidgetConfig::CONTEXT_DASHBOARD;
+	}
+
 	/**
 	 * Set validation rules for input parameters.
 	 *
@@ -82,12 +88,12 @@ abstract class CControllerWidget extends CController {
 	}
 
 	/**
-	 * Returns default widget header.
+	 * Returns default widget name.
 	 *
 	 * @return string
 	 */
-	protected function getDefaultHeader() {
-		return CWidgetConfig::getKnownWidgetTypes()[$this->type];
+	protected function getDefaultName() {
+		return CWidgetConfig::getKnownWidgetTypes($this->getContext())[$this->type];
 	}
 
 	/**
@@ -96,14 +102,22 @@ abstract class CControllerWidget extends CController {
 	 * @return bool
 	 */
 	protected function checkInput() {
-		$ret = $this->validateInput($this->validation_rules);
+		$validation_rules = $this->validation_rules;
+
+		if (CWidgetConfig::isWidgetTypeSupportedInContext($this->type, CWidgetConfig::CONTEXT_TEMPLATE_DASHBOARD)) {
+			$validation_rules['templateid'] = 'db dashboard.templateid';
+		}
+
+		$ret = $this->validateInput($validation_rules);
 
 		if ($ret) {
-			$this->form = CWidgetConfig::getForm($this->type, $this->getInput('fields', '{}'));
+			$this->form = CWidgetConfig::getForm($this->type, $this->getInput('fields', '{}'),
+				$this->hasInput('templateid') ? $this->getInput('templateid') : null
+			);
 
 			if ($errors = $this->form->validate()) {
 				foreach ($errors as $error) {
-					info($error);
+					error($error);
 				}
 
 				$ret = false;
@@ -112,15 +126,12 @@ abstract class CControllerWidget extends CController {
 
 		if (!$ret) {
 			$output = [
-				'header' => $this->getDefaultHeader()
+				'name' => $this->getDefaultName(),
+				'messages' => getMessages()->toString()
 			];
 
-			if (($messages = getMessages()) !== null) {
-				$output['messages'] = $messages->toString();
-			}
-
 			$this->setResponse(
-				(new CControllerResponseData(['main_block' => (new CJson())->encode($output)]))->disableView()
+				(new CControllerResponseData(['main_block' => json_encode($output)]))->disableView()
 			);
 		}
 
