@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,7 +27,9 @@ $this->addJsFile('gtlc.js');
 $this->addJsFile('flickerfreescreen.js');
 $this->addJsFile('layout.mode.js');
 $this->addJsFile('multiselect.js');
+$this->addJsFile('class.tagfilteritem.js');
 
+$this->includeJsFile('monitoring.web.view.js.php');
 
 $this->enableLayoutModes();
 $web_layout_mode = $this->getLayoutMode();
@@ -58,7 +60,8 @@ $web_layout_mode = $this->getLayoutMode();
 								'dstfrm' => 'zbx_filter',
 								'dstfld1' => 'filter_groupids_',
 								'with_httptests' => true,
-								'real_hosts' => true
+								'real_hosts' => true,
+								'enrich_parent_groups' => true
 							]
 						]
 					]))->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
@@ -80,7 +83,13 @@ $web_layout_mode = $this->getLayoutMode();
 							]
 						]
 					]))->setWidth(ZBX_TEXTAREA_FILTER_STANDARD_WIDTH)
-				)
+				),
+			(new CFormList())->addRow(_('Tags'),
+				CTagFilterFieldHelper::getTagFilterField([
+					'evaltype' => $data['filter']['evaltype'],
+					'tags' => $data['filter']['tags']
+				])
+			)
 		])
 	)
 	->addItem($data['screen_view'])
