@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -78,7 +78,6 @@ class CHostGroup extends CApiService {
 			'with_monitored_httptests'				=> null,
 			'with_graphs'							=> null,
 			'with_graph_prototypes'					=> null,
-			'with_applications'						=> null,
 			'editable'								=> false,
 			'nopermissions'							=> null,
 			// filter
@@ -293,12 +292,6 @@ class CHostGroup extends CApiService {
 			);
 		}
 
-		// with_applications
-		if ($options['with_applications'] !== null) {
-			$sub_sql_parts['from']['a'] = 'applications a';
-			$sub_sql_parts['where']['hg-a'] = 'hg.hostid=a.hostid';
-		}
-
 		if ($sub_sql_parts) {
 			$sub_sql_parts['from']['hg'] = 'hosts_groups hg';
 			$sub_sql_parts['where']['g-hg'] = 'g.groupid=hg.groupid';
@@ -599,19 +592,6 @@ class CHostGroup extends CApiService {
 	public function delete(array $groupids, $nopermissions = false) {
 		$this->validateDelete($groupids, $db_groups, $nopermissions);
 
-		// delete screens items
-		$resources = [
-			SCREEN_RESOURCE_HOSTGROUP_TRIGGERS,
-			SCREEN_RESOURCE_HOST_INFO,
-			SCREEN_RESOURCE_TRIGGER_INFO,
-			SCREEN_RESOURCE_TRIGGER_OVERVIEW,
-			SCREEN_RESOURCE_DATA_OVERVIEW
-		];
-		DB::delete('screens_items', [
-			'resourceid' => $groupids,
-			'resourcetype' => $resources
-		]);
-
 		// delete sysmap element
 		if (!empty($groupids)) {
 			DB::delete('sysmaps_elements', ['elementtype' => SYSMAP_ELEMENT_TYPE_HOST_GROUP, 'elementid' => $groupids]);
@@ -739,7 +719,8 @@ class CHostGroup extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
 		}
 
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['name']], 'fields' => [
+			'uuid' =>	['type' => API_UUID],
 			'name' =>	['type' => API_HG_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hstgrp', 'name')]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
@@ -747,6 +728,35 @@ class CHostGroup extends CApiService {
 		}
 
 		$this->checkDuplicates(zbx_objectValues($groups, 'name'));
+		$this->checkAndAddUuid($groups);
+	}
+
+	/**
+	 * Check that new UUIDs are not already used and generate UUIDs where missing.
+	 *
+	 * @param array $groups_to_create
+	 *
+	 * @throws APIException
+	 */
+	protected function checkAndAddUuid(array &$groups_to_create): void {
+		foreach ($groups_to_create as &$group) {
+			if (!array_key_exists('uuid', $group)) {
+				$group['uuid'] = generateUuidV4();
+			}
+		}
+		unset($group);
+
+		$db_uuid = DB::select('hstgrp', [
+			'output' => ['uuid'],
+			'filter' => ['uuid' => array_column($groups_to_create, 'uuid')],
+			'limit' => 1
+		]);
+
+		if ($db_uuid) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Entry with UUID "%1$s" already exists.', $db_uuid[0]['uuid'])
+			);
+		}
 	}
 
 	/**
@@ -1409,15 +1419,21 @@ class CHostGroup extends CApiService {
 		// adding hosts
 		if ($options['selectHosts'] !== null) {
 			if ($options['selectHosts'] !== API_OUTPUT_COUNT) {
+				$hosts = [];
 				$relationMap = $this->createRelationMap($result, 'groupid', 'hostid', 'hosts_groups');
-				$hosts = API::Host()->get([
-					'output' => $options['selectHosts'],
-					'hostids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				if (!is_null($options['limitSelects'])) {
-					order_result($hosts, 'host');
+				$related_ids = $relationMap->getRelatedIds();
+
+				if ($related_ids) {
+					$hosts = API::Host()->get([
+						'output' => $options['selectHosts'],
+						'hostids' => $related_ids,
+						'preservekeys' => true
+					]);
+					if (!is_null($options['limitSelects'])) {
+						order_result($hosts, 'host');
+					}
 				}
+
 				$result = $relationMap->mapMany($result, $hosts, 'hosts', $options['limitSelects']);
 			}
 			else {
@@ -1438,15 +1454,21 @@ class CHostGroup extends CApiService {
 		// adding templates
 		if ($options['selectTemplates'] !== null) {
 			if ($options['selectTemplates'] !== API_OUTPUT_COUNT) {
+				$hosts = [];
 				$relationMap = $this->createRelationMap($result, 'groupid', 'hostid', 'hosts_groups');
-				$hosts = API::Template()->get([
-					'output' => $options['selectTemplates'],
-					'templateids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				if (!is_null($options['limitSelects'])) {
-					order_result($hosts, 'host');
+				$related_ids = $relationMap->getRelatedIds();
+
+				if ($related_ids) {
+					$hosts = API::Template()->get([
+						'output' => $options['selectTemplates'],
+						'templateids' => $related_ids,
+						'preservekeys' => true
+					]);
+					if (!is_null($options['limitSelects'])) {
+						order_result($hosts, 'host');
+					}
 				}
+
 				$result = $relationMap->mapMany($result, $hosts, 'templates', $options['limitSelects']);
 			}
 			else {
