@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -42,7 +42,6 @@ class CUserGroup extends CApiService {
 	 * @param array  $options['usrgrpids']
 	 * @param array  $options['userids']
 	 * @param bool   $options['status']
-	 * @param bool   $options['with_gui_access']
 	 * @param bool   $options['selectUsers']
 	 * @param int    $options['count']
 	 * @param string $options['pattern']
@@ -66,7 +65,6 @@ class CUserGroup extends CApiService {
 			'usrgrpids'					=> null,
 			'userids'					=> null,
 			'status'					=> null,
-			'with_gui_access'			=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> null,
@@ -124,11 +122,6 @@ class CUserGroup extends CApiService {
 			$sqlParts['where'][] = 'g.users_status='.zbx_dbstr($options['status']);
 		}
 
-		// with_gui_access
-		if (!is_null($options['with_gui_access'])) {
-			$sqlParts['where'][] = 'g.gui_access='.GROUP_GUI_ACCESS_ENABLED;
-		}
-
 		// filter
 		if (is_array($options['filter'])) {
 			$this->dbFilter('usrgrp g', $options, $sqlParts);
