@@ -1,7 +1,7 @@
 <?php declare(strict_types = 1);
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -99,7 +99,8 @@ class CSettingsHelper extends CConfigGeneralHelper {
 				: API::Settings()->get(['output' => 'extend']);
 		}
 		else if (!$is_global && $param && !array_key_exists($param, self::$params)) {
-			self::$params = API::Settings()->get(['output' => 'extend']) + self::$params;
+			$settings = API::Settings()->get(['output' => 'extend']);
+			self::$params = is_array($settings) ? ($settings + self::$params) : false;
 		}
 
 		if (self::$params === false) {
