@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ class CLdap {
 			'userfilter' => '(%{attr}=%{user})',
 			'groupkey' => 'cn',
 			'mapping' => [
-				'alias' => 'uid',
+				'username' => 'uid',
 				'userid' => 'uidnumbera',
 				'passwd' => 'userpassword'
 			],
@@ -93,7 +93,7 @@ class CLdap {
 			return false;
 		}
 
-		// set protocol version and dependend options
+		// Set protocol version and dependent options.
 		if ($this->cnf['version']) {
 			if (!@ldap_set_option($this->ds, LDAP_OPT_PROTOCOL_VERSION, $this->cnf['version'])) {
 				$this->error = static::ERR_OPT_PROTOCOL_FAILED;
@@ -122,6 +122,8 @@ class CLdap {
 
 	public function checkPass($user, $pass) {
 		if (!$pass) {
+			$this->error = static::ERR_USER_NOT_FOUND;
+
 			return false;
 		}
 
