@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -59,7 +59,10 @@ class CScreenHttpTest extends CScreenBase {
 		$options = [
 			'output' => ['httptestid', 'name', 'hostid'],
 			'selectHosts' => ['name', 'status'],
+			'selectTags' => ['tag', 'value'],
 			'selectSteps' => API_OUTPUT_COUNT,
+			'evaltype' => array_key_exists('evaltype', $this->data) ? $this->data['evaltype'] : TAG_EVAL_TYPE_AND_OR,
+			'tags' => array_key_exists('tags', $this->data) ? $this->data['tags'] : [],
 			'templated' => false,
 			'preservekeys' => true,
 			'filter' => ['status' => HTTPTEST_STATUS_ACTIVE],
@@ -87,6 +90,10 @@ class CScreenHttpTest extends CScreenBase {
 		$httptests = resolveHttpTestMacros($httptests, true, false);
 		order_result($httptests, $sort_field, $sort_order);
 
+		$tags = array_key_exists('tags', $this->data)
+			? makeTags($httptests, true, 'httptestid', ZBX_TAG_COUNT_DEFAULT, $this->data['tags'])
+			: [];
+
 		// Fetch the latest results of the web scenario.
 		$last_httptest_data = Manager::HttpTest()->getLastData(array_keys($httptests));
 
@@ -106,10 +113,11 @@ class CScreenHttpTest extends CScreenBase {
 				make_sorting_header(_('Name'), 'name', $sort_field, $sort_order, 'zabbix.php?action=web.view'),
 				_('Number of steps'),
 				_('Last check'),
-				_('Status')
+				_('Status'),
+				_('Tags')
 			]);
 
-		foreach ($httptests as $httptest) {
+		foreach ($httptests as $key => $httptest) {
 			if (array_key_exists('lastfailedstep', $httptest) && $httptest['lastfailedstep'] !== null) {
 				$lastcheck = zbx_date2str(DATE_TIME_FORMAT_SECONDS, $httptest['lastcheck']);
 
@@ -142,7 +150,8 @@ class CScreenHttpTest extends CScreenBase {
 				new CLink($httptest['name'], 'httpdetails.php?httptestid='.$httptest['httptestid']),
 				$httptest['steps'],
 				$lastcheck,
-				$status
+				$status,
+				array_key_exists($key, $tags) ? $tags[$key] : ''
 			]));
 		}
 
