@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -70,6 +70,9 @@ class CApiInputValidator {
 	 */
 	private static function validateData($rule, &$data, $path, &$error, array $parent_data = null) {
 		switch ($rule['type']) {
+			case API_CALC_FORMULA:
+				return self::validateCalcFormula($rule, $data, $path, $error);
+
 			case API_COLOR:
 				return self::validateColor($rule, $data, $path, $error);
 
@@ -139,8 +142,8 @@ class CApiInputValidator {
 			case API_NUMERIC:
 				return self::validateNumeric($rule, $data, $path, $error);
 
-			case API_SCRIPT_NAME:
-				return self::validateScriptName($rule, $data, $path, $error);
+			case API_SCRIPT_MENU_PATH:
+				return self::validateScriptMenuPath($rule, $data, $path, $error);
 
 			case API_USER_MACRO:
 				return self::validateUserMacro($rule, $data, $path, $error);
@@ -168,6 +171,39 @@ class CApiInputValidator {
 
 			case API_URL:
 				return self::validateUrl($rule, $data, $path, $error);
+
+			case API_IP:
+				return self::validateIp($rule, $data, $path, $error);
+
+			case API_DNS:
+				return self::validateDns($rule, $data, $path, $error);
+
+			case API_PORT:
+				return self::validatePort($rule, $data, $path, $error);
+
+			case API_TRIGGER_EXPRESSION:
+				return self::validateTriggerExpression($rule, $data, $path, $error);
+
+			case API_EVENT_NAME:
+				return self::validateEventName($rule, $data, $path, $error);
+
+			case API_JSONRPC_PARAMS:
+				return self::validateJsonRpcParams($rule, $data, $path, $error);
+
+			case API_JSONRPC_ID:
+				return self::validateJsonRpcId($rule, $data, $path, $error);
+
+			case API_DATE:
+				return self::validateDate($rule, $data, $path, $error);
+
+			case API_NUMERIC_RANGES:
+				return self::validateNumericRanges($rule, $data, $path, $error);
+
+			case API_UUID:
+				return self::validateUuid($rule, $data, $path, $error);
+
+			case API_VAULT_SECRET:
+				return self::validateVaultSecret($rule, $data, $path, $error);
 		}
 
 		// This message can be untranslated because warn about incorrect validation rules at a development stage.
@@ -188,6 +224,7 @@ class CApiInputValidator {
 	 */
 	private static function validateDataUniqueness($rule, &$data, $path, &$error) {
 		switch ($rule['type']) {
+			case API_CALC_FORMULA:
 			case API_COLOR:
 			case API_MULTIPLE:
 			case API_STRING_UTF8:
@@ -205,7 +242,7 @@ class CApiInputValidator {
 			case API_HG_NAME:
 			case API_H_NAME:
 			case API_NUMERIC:
-			case API_SCRIPT_NAME:
+			case API_SCRIPT_MENU_PATH:
 			case API_USER_MACRO:
 			case API_LLD_MACRO:
 			case API_RANGE_TIME:
@@ -215,6 +252,17 @@ class CApiInputValidator {
 			case API_HTTP_POST:
 			case API_VARIABLE_NAME:
 			case API_URL:
+			case API_IP:
+			case API_DNS:
+			case API_PORT:
+			case API_TRIGGER_EXPRESSION:
+			case API_EVENT_NAME:
+			case API_JSONRPC_PARAMS:
+			case API_JSONRPC_ID:
+			case API_DATE:
+			case API_NUMERIC_RANGES:
+			case API_UUID:
+			case API_VAULT_SECRET:
 				return true;
 
 			case API_OBJECT:
@@ -272,6 +320,57 @@ class CApiInputValidator {
 		return true;
 	}
 
+	/**
+	 * Calculated item formula validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional) API_ALLOW_LLD_MACRO
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateCalcFormula($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8(API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$expression_parser = new CExpressionParser([
+			'usermacros' => true,
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO),
+			'calculated' => true,
+			'host_macro' => true,
+			'empty_host' => true
+		]);
+
+		if ($expression_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $expression_parser->getError());
+			return false;
+		}
+
+		$expression_validator = new CExpressionValidator([
+			'usermacros' => true,
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO),
+			'calculated' => true
+		]);
+
+		if (!$expression_validator->validate($expression_parser->getResult()->getTokens())) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $expression_validator->getError());
+			return false;
+		}
+
+		return true;
+	}
+
 	/**
 	 * Color validator.
 	 *
@@ -1083,6 +1182,7 @@ class CApiInputValidator {
 	 * @param array  $rule
 	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY, API_ALLOW_NULL, API_NORMALIZE, API_PRESERVE_KEYS
 	 * @param array  $rule['fields']
+	 * @param int    $rule['length']  (optional)
 	 * @param mixed  $data
 	 * @param string $path
 	 * @param string $error
@@ -1106,6 +1206,11 @@ class CApiInputValidator {
 			return false;
 		}
 
+		if (array_key_exists('length', $rule) && count($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
 		if (($flags & API_NORMALIZE) && $data) {
 			reset($data);
 
@@ -1293,9 +1398,10 @@ class CApiInputValidator {
 	}
 
 	/**
-	 * Global script name validator.
+	 * Global script menu_path validator.
 	 *
 	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional)
 	 * @param int    $rule['length']  (optional)
 	 * @param mixed  $data
 	 * @param string $path
@@ -1303,8 +1409,17 @@ class CApiInputValidator {
 	 *
 	 * @return bool
 	 */
-	private static function validateScriptName($rule, &$data, $path, &$error) {
-		if (self::checkStringUtf8(API_NOT_EMPTY, $data, $path, $error) === false) {
+	private static function validateScriptMenuPath($rule, &$data, $path, &$error) {
+		// Having only a root folder is the same as being empty. Temporary modify data to check if it is actually empty.
+		$tmp_data = $data;
+
+		if ($tmp_data === '/') {
+			$tmp_data = '';
+		}
+
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags, $tmp_data, $path, $error) === false) {
 			return false;
 		}
 
@@ -1313,13 +1428,21 @@ class CApiInputValidator {
 			return false;
 		}
 
+		// If empty is allowed there is only root folder, return early.
+		if ($data === '/') {
+			return true;
+		}
+
 		$folders = splitPath($data);
 		$folders = array_map('trim', $folders);
+		$count = count($folders);
 
 		// folder1/{empty}/name or folder1/folder2/{empty}
-		foreach ($folders as $folder) {
-			if ($folder === '') {
-				$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('directory or script name cannot be empty'));
+		foreach ($folders as $num => $folder) {
+			// Allow the trailing slash.
+			if ($folder === '' && $num != ($count - 1) && $num != 0) {
+				$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('directory cannot be empty'));
+
 				return false;
 			}
 		}
@@ -1348,8 +1471,10 @@ class CApiInputValidator {
 			return false;
 		}
 
-		if ((new CUserMacroParser())->parse($data) != CParser::PARSE_SUCCESS) {
-			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a user macro is expected'));
+		$user_macro_parser = new CUserMacroParser();
+
+		if ($user_macro_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $user_macro_parser->getError());
 			return false;
 		}
 
@@ -1456,7 +1581,7 @@ class CApiInputValidator {
 	 * Regular expression validator.
 	 *
 	 * @param array  $rule
-	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY
+	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY, API_ALLOW_GLOBAL_REGEX
 	 * @param int    $rule['length']  (optional)
 	 * @param mixed  $data
 	 * @param string $path
@@ -1476,7 +1601,7 @@ class CApiInputValidator {
 			return false;
 		}
 
-		if ($data !== '' && $data[0] === '@') {
+		if (($flags & API_ALLOW_GLOBAL_REGEX) && $data !== '' && $data[0] === '@') {
 			return true;
 		}
 
@@ -1492,6 +1617,7 @@ class CApiInputValidator {
 	 * Time unit validator like "10", "20s", "30m", "4h", "{$TIME}" etc.
 	 *
 	 * @param array  $rule
+	 * @param int    $rule['length'] (optional)
 	 * @param int    $rule['flags']  (optional) API_NOT_EMPTY, API_ALLOW_USER_MACRO, API_ALLOW_LLD_MACRO,
 	 *                                          API_TIME_UNIT_WITH_YEAR
 	 * @param int    $rule['in']     (optional)
@@ -1516,6 +1642,11 @@ class CApiInputValidator {
 			return false;
 		}
 
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
 		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
 			return true;
 		}
@@ -1536,7 +1667,7 @@ class CApiInputValidator {
 			return true;
 		}
 
-		$seconds = timeUnitToSeconds($data);
+		$seconds = timeUnitToSeconds($data, ($flags & API_TIME_UNIT_WITH_YEAR));
 
 		if ($seconds < ZBX_MIN_INT32 || $seconds > ZBX_MAX_INT32) {
 			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a number is too large'));
@@ -1583,12 +1714,44 @@ class CApiInputValidator {
 		return true;
 	}
 
+	/**
+	 * Returns macro without spaces and curly braces.
+	 *
+	 * "{$MACRO}" => "MACRO"
+	 * "{$MACRO:}" => "MACRO:context:"
+	 * "{$MACRO: /var}" => "MACRO:context:/var"
+	 * "{$MACRO: /"var"}" => "MACRO:context:/var"
+	 * "{$MACRO:regex: ^[a-z]+}" => "MACRO:regex:^[a-z]+"
+	 *
+	 * @param string $macro
+	 *
+	 * @return string
+	 */
+	public static function trimMacro(string $macro): string {
+		$user_macro_parser = new CUserMacroParser();
+
+		$user_macro_parser->parse($macro);
+
+		$macro = $user_macro_parser->getMacro();
+		$context = $user_macro_parser->getContext();
+		$regex = $user_macro_parser->getRegex();
+
+		if ($context !== null) {
+			$macro .= ':context:'.$context;
+		}
+		elseif ($regex !== null) {
+			$macro .= ':regex:'.$regex;
+		}
+
+		return $macro;
+	}
+
 	/**
 	 * Array of objects uniqueness validator.
 	 *
 	 * @param array  $rule
 	 * @param array  $rule['uniq']    (optional) subsets of unique fields ([['hostid', 'name'], [...]])
-	 * @param array  $rule['fields']  (optional)
+	 * @param array  $rule['fields']
 	 * @param array  $data
 	 * @param string $path
 	 * @param string $error
@@ -1617,15 +1780,19 @@ class CApiInputValidator {
 
 						$values[] = $object[$field_name];
 
+						$value = ($rule['fields'][$field_name]['type'] == API_USER_MACRO)
+							? self::trimMacro($object[$field_name])
+							: $object[$field_name];
+
 						if ($level < count($field_names)) {
-							if (!array_key_exists($object[$field_name], $_uniq)) {
-								$_uniq[$object[$field_name]] = [];
+							if (!array_key_exists($value, $_uniq)) {
+								$_uniq[$value] = [];
 							}
 
-							$_uniq = &$_uniq[$object[$field_name]];
+							$_uniq = &$_uniq[$value];
 						}
 						else {
-							if (array_key_exists($object[$field_name], $_uniq)) {
+							if (array_key_exists($value, $_uniq)) {
 								$subpath = ($path === '/' ? $path : $path.'/').($index + 1);
 								$error = _s('Invalid parameter "%1$s": %2$s.', $subpath, _s('value %1$s already exists',
 									'('.implode(', ', $field_names).')=('.implode(', ', $values).')'
@@ -1633,7 +1800,7 @@ class CApiInputValidator {
 								return false;
 							}
 
-							$_uniq[$object[$field_name]] = true;
+							$_uniq[$value] = true;
 						}
 
 						$level++;
@@ -1642,14 +1809,12 @@ class CApiInputValidator {
 			}
 		}
 
-		if (array_key_exists('fields', $rule)) {
-			foreach ($data as $index => $object) {
-				foreach ($rule['fields'] as $field_name => $field_rule) {
-					if (array_key_exists($field_name, $object)) {
-						$subpath = ($path === '/' ? $path : $path.'/').($index + 1).'/'.$field_name;
-						if (!self::validateDataUniqueness($field_rule, $object[$field_name], $subpath, $error)) {
-							return false;
-						}
+		foreach ($data as $index => $object) {
+			foreach ($rule['fields'] as $field_name => $field_rule) {
+				if (array_key_exists($field_name, $object)) {
+					$subpath = ($path === '/' ? $path : $path.'/').($index + 1).'/'.$field_name;
+					if (!self::validateDataUniqueness($field_rule, $object[$field_name], $subpath, $error)) {
+						return false;
 					}
 				}
 			}
@@ -1762,4 +1927,422 @@ class CApiInputValidator {
 
 		return true;
 	}
+
+	/**
+	 * IP address validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY, API_ALLOW_USER_MACRO, API_ALLOW_LLD_MACRO,
+	 *                                API_ALLOW_MACRO
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateIp($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$ip_parser = new CIPParser([
+			'v6' => ZBX_HAVE_IPV6,
+			'usermacros' => ($flags & API_ALLOW_USER_MACRO),
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO),
+			'macros' => ($flags & API_ALLOW_MACRO)
+		]);
+
+		if ($ip_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('an IP address is expected'));
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * DNS name validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY, API_ALLOW_USER_MACRO, API_ALLOW_LLD_MACRO,
+	 *                                API_ALLOW_MACRO
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateDns($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$dns_parser = new CDnsParser([
+			'usermacros' => ($flags & API_ALLOW_USER_MACRO),
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO),
+			'macros' => ($flags & API_ALLOW_MACRO)
+		]);
+
+		if ($dns_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a DNS name is expected'));
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * Port number validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY, API_ALLOW_USER_MACRO, API_ALLOW_LLD_MACRO
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validatePort($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (!is_int($data) && !is_string($data)) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a number is expected'));
+			return false;
+		}
+
+		$data = (string) $data;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		if ($flags & API_ALLOW_USER_MACRO) {
+			$user_macro_parser = new CUserMacroParser();
+
+			if ($user_macro_parser->parse($data) == CParser::PARSE_SUCCESS) {
+				return true;
+			}
+		}
+
+		if ($flags & API_ALLOW_LLD_MACRO) {
+			$lld_macro_parser = new CLLDMacroParser();
+
+			if ($lld_macro_parser->parse($data) == CParser::PARSE_SUCCESS) {
+				return true;
+			}
+		}
+
+		if (!self::validateInt32(['in' => ZBX_MIN_PORT_NUMBER.':'.ZBX_MAX_PORT_NUMBER], $data, $path, $error)) {
+			return false;
+		}
+
+		$data = (string) $data;
+
+		return true;
+	}
+
+	/**
+	 * Trigger expression validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['length']  (optional)
+	 * @param int    $rule['flags']   (optional) API_ALLOW_LLD_MACRO, API_NOT_EMPTY
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateTriggerExpression($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$expression_parser = new CExpressionParser([
+			'usermacros' => true,
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO)
+		]);
+
+		if ($expression_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $expression_parser->getError());
+			return false;
+		}
+
+		$expression_validator = new CExpressionValidator([
+			'usermacros' => true,
+			'lldmacros' => ($flags & API_ALLOW_LLD_MACRO)
+		]);
+
+		if (!$expression_validator->validate($expression_parser->getResult()->getTokens())) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $expression_validator->getError());
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * Event name validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateEventName($rule, &$data, $path, &$error) {
+		if (self::checkStringUtf8(0, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$eventname_validator = new CEventNameValidator();
+
+		if (!$eventname_validator->validate($data)) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $eventname_validator->getError());
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * JSON RPC parameters validator. Parameters MUST contain an array or object value.
+	 *
+	 * @param array  $rule
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateJsonRpcParams($rule, &$data, $path, &$error) {
+		if (is_array($data)) {
+			return true;
+		}
+
+		$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('an array or object is expected'));
+
+		return false;
+	}
+
+	/**
+	 * JSON RPC identifier validator. This identifier MUST contain a String, Number, or NULL value.
+	 *
+	 * @param array  $rule
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateJsonRpcId($rule, &$data, $path, &$error) {
+		if (is_string($data) || is_int($data) || is_float($data) || $data === null) {
+			return true;
+		}
+
+		$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a string, number or null value is expected'));
+
+		return false;
+	}
+
+	/**
+	 * Date validator in YYYY-MM-DD format.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']  (optional) API_NOT_EMPTY
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateDate(array $rule, &$data, string $path, string &$error): bool {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		[$year, $month, $day] = sscanf($data, '%d-%d-%d');
+
+		if (!checkdate($month, $day, $year)) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('a date in YYYY-MM-DD format is expected'));
+			return false;
+		}
+
+		if (!validateDateInterval($year, $month, $day)) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path,
+				_s('value must be between "%1$s" and "%2$s"', '1970-01-01', '2038-01-18')
+			);
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * Validate numeric ranges. Multiple ranges separated by comma character.
+	 * Example:
+	 *   10-20,-20--10,-5-0,0.5-0.7,-20--10,-20.20--20.10
+	 *   30,-10,0.7,-0.5
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']   (optional) API_NOT_EMPTY
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateNumericRanges($rule, &$data, $path, &$error) {
+		$flags = array_key_exists('flags', $rule) ? $rule['flags'] : 0x00;
+
+		if (self::checkStringUtf8($flags & API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (($flags & API_NOT_EMPTY) == 0 && $data === '') {
+			return true;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$parser = new CRangesParser(['with_minus' => true, 'with_float' => true, 'with_suffix' => true]);
+
+		if ($parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('invalid range expression'));
+
+			return false;
+		}
+
+		return true;
+	}
+
+	/**
+	 * UUIDv4 validator.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['flags']  (optional) API_NOT_EMPTY
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateUuid(array $rule, &$data, string $path, string &$error): bool {
+		if (self::checkStringUtf8(API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (mb_strlen($data) != 32) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _s('must be %1$s characters long', 32));
+			return false;
+		}
+
+		if (!ctype_xdigit($data)) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('UUIDv4 is expected'));
+			return false;
+		}
+
+		$binary = hex2bin($data);
+		if ((ord($binary[6]) & 0xf0) != 0x40 || (ord($binary[8]) & 0xc0) != 0x80) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('UUIDv4 is expected'));
+			return false;
+		}
+
+		$data = strtolower($data);
+
+		return true;
+	}
+
+	/**
+	 * User vault secret.
+	 *
+	 * @param array  $rule
+	 * @param int    $rule['length']  (optional)
+	 * @param mixed  $data
+	 * @param string $path
+	 * @param string $error
+	 *
+	 * @return bool
+	 */
+	private static function validateVaultSecret($rule, &$data, $path, &$error) {
+		if (self::checkStringUtf8(API_NOT_EMPTY, $data, $path, $error) === false) {
+			return false;
+		}
+
+		if (array_key_exists('length', $rule) && mb_strlen($data) > $rule['length']) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, _('value is too long'));
+			return false;
+		}
+
+		$vault_secret_parser = new CVaultSecretParser();
+
+		if ($vault_secret_parser->parse($data) != CParser::PARSE_SUCCESS) {
+			$error = _s('Invalid parameter "%1$s": %2$s.', $path, $vault_secret_parser->getError());
+			return false;
+		}
+
+		return true;
+	}
 }
