@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,41 +22,45 @@
  * Automatic checkbox range selection
  */
 var chkbxRange = {
-	startbox:		null,	// start checkbox obj
-	chkboxes:		{},		// ckbx list
-	prefix:			null,	// prefix for cookie name
-	pageGoName:		null,	// which checkboxes should be counted by Go button and saved to cookies
-	selectedIds:	{},		// ids of selected objects
-	footerButtons:	{},		// action buttons at the bottom of page
-	cookieName:		null,
+	startbox:			null,	// start checkbox obj
+	chkboxes:			{},		// ckbx list
+	prefix:				null,	// prefix for session storage variable name
+	pageGoName:			null,	// which checkboxes should be counted by Go button and saved to session storage
+	selectedIds:		{},		// ids of selected objects
+	footerButtons:		{},		// action buttons at the bottom of page
+	sessionStorageName:	null,
 
 	init: function() {
-		// cookie name
 		var path = new Curl();
 		var filename = basename(path.getPath(), '.php');
-		this.cookieName = 'cb_' + filename + (this.prefix ? '_' + this.prefix : '');
+		this.sessionStorageName = 'cb_' + filename + (this.prefix ? '_' + this.prefix : '');
+		// Erase old checkboxes.
+		this.chkboxes = {};
+		this.startbox = null;
 
-		this.resetOtherPageCookies();
+		this.resetOtherPage();
 
 		// initialize checkboxes
-		var chkboxes = jQuery('.list-table input[type=checkbox]:not(:disabled)');
+		var chkboxes = jQuery('.list-table tbody input[type=checkbox]:not(:disabled)');
 		if (chkboxes.length > 0) {
 			for (var i = 0; i < chkboxes.length; i++) {
 				this.implement(chkboxes[i]);
 			}
 		}
 
-		// load selected checkboxes from cookies or cache
+		// load selected checkboxes from session storage or cache
 		if (this.pageGoName != null) {
-			this.selectedIds = cookie.readJSON(this.cookieName);
+			this.selectedIds = sessionStorage.getItem(this.sessionStorageName) === null
+				? {}
+				: JSON.parse(sessionStorage.getItem(this.sessionStorageName));
 
-			// check if checkboxes should be selected from cookies
+			// check if checkboxes should be selected from session storage
 			if (!jQuery.isEmptyObject(this.selectedIds)) {
 				var objectIds = jQuery.map(this.selectedIds, function(id) { return id });
 			}
-			// no checkboxes selected from cookies, check browser cache if checkboxes are still checked and update state
+			// no checkboxes selected, check browser cache if checkboxes are still checked and update state
 			else {
-				var checkedFromCache = jQuery('.list-table tr:not(.header) input[type=checkbox]:checked:not(:disabled)');
+				var checkedFromCache = jQuery('main .list-table tbody input[type=checkbox]:checked:not(:disabled)');
 				var objectIds = jQuery.map(checkedFromCache, jQuery.proxy(function(checkbox) {
 					return this.getObjectIdFromName(checkbox.name);
 				}, this));
@@ -103,7 +107,7 @@ var chkbxRange = {
 	 */
 	handleClick: function(e) {
 		e = e || window.event;
-		var checkbox = Event.element(e);
+		var checkbox = e.target;
 
 		PageRefresh.restart();
 
@@ -120,7 +124,7 @@ var chkbxRange = {
 		}
 
 		this.update(object);
-		this.saveCookies(object);
+		this.saveSessionStorage(object);
 
 		this.startbox = checkbox;
 	},
@@ -212,7 +216,6 @@ var chkbxRange = {
 			objectIds.push(this.getObjectIdFromName(checkboxes[i].name));
 		}
 		this.checkObjects(object, objectIds, checked);
-
 	},
 
 	/**
@@ -257,7 +260,13 @@ var chkbxRange = {
 		var selectedCountSpan = jQuery('#selected_count');
 		selectedCountSpan.text(count + ' ' + selectedCountSpan.text().split(' ')[1]);
 
-		jQuery('#action_buttons button').prop('disabled', count == 0);
+		jQuery('#action_buttons button').each((_, val) => {
+			const $val = jQuery(val);
+
+			if (!$val.data('disabled')) {
+				$val.prop('disabled', count == 0);
+			}
+		});
 	},
 
 	// check if all checkboxes are selected and select main checkbox, else disable checkbox, select options and button
@@ -288,28 +297,31 @@ var chkbxRange = {
 	},
 
 	/**
-	 * Save the state of the checkboxes belonging to the given object group in cookies.
+	 * Save the state of the checkboxes belonging to the given object group in SessionStorage.
 	 *
 	 * @param {string} object
 	 */
-	saveCookies: function(object) {
+	saveSessionStorage: function(object) {
 		if (this.pageGoName == object) {
-			cookie.createJSON(this.cookieName, this.selectedIds);
+			sessionStorage.setItem(this.sessionStorageName, JSON.stringify(this.selectedIds));
 		}
 	},
 
 	clearSelectedOnFilterChange: function() {
-		cookie.eraseArray(this.cookieName);
+		sessionStorage.removeItem(this.sessionStorageName);
 	},
 
 	/**
 	 * Reset all selections on other pages.
 	 */
-	resetOtherPageCookies: function() {
-		for (var key in cookie.cookies) {
-			var cookiePair = key.split('=');
-			if (cookiePair[0].indexOf('cb_') > -1 && cookiePair[0].indexOf(this.cookieName) == -1) {
-				cookie.erase(key);
+	resetOtherPage: function() {
+		var key_;
+
+		for (var i = 0; i < sessionStorage.length; i++) {
+			key_ = sessionStorage.key(i);
+
+			if (key_.substring(0, 3) === 'cb_' && key_ != this.sessionStorageName) {
+				sessionStorage.removeItem(key_);
 			}
 		}
 	},
@@ -321,7 +333,9 @@ var chkbxRange = {
 			form = footerButton.closest('form'),
 			confirmText = footerButton.attr('confirm');
 		if (confirmText && !confirm(confirmText)) {
-			Event.stop(e);
+			e.preventDefault();
+			e.stopPropagation();
+
 			return false;
 		}
 
