@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,6 +18,21 @@
 **/
 
 
+const KEY_ARROW_DOWN = 40;
+const KEY_ARROW_LEFT = 37;
+const KEY_ARROW_RIGHT = 39;
+const KEY_ARROW_UP = 38;
+const KEY_BACKSPACE = 8;
+const KEY_DELETE = 46;
+const KEY_ENTER = 13;
+const KEY_ESCAPE = 27;
+const KEY_TAB = 9;
+const KEY_PAGE_UP = 33;
+const KEY_PAGE_DOWN = 34;
+const KEY_END = 35;
+const KEY_HOME = 36;
+const KEY_SPACE = 32;
+
 /**
  * jQuery based publish/subscribe handler.
  *
@@ -311,26 +326,29 @@ function getPosition(obj) {
 /**
  * Opens popup content in overlay dialogue.
  *
- * @param {string} action         Popup controller related action.
- * @param {array}  options        (optional) Array with key/value pairs that will be used as query for popup request.
- * @param {string} dialogueid     (optional) id of overlay dialogue.
- * @param {object} trigger_elmnt  (optional) UI element which was clicked to open overlay dialogue.
+ * @param {string} action          Popup controller related action.
+ * @param {array|object}  options  (optional) Array with key/value pairs that will be used as query for popup request.
+ * @param {string} dialogueid      (optional) id of overlay dialogue.
+ * @param {object} trigger_elmnt   (optional) UI element which was clicked to open overlay dialogue.
  *
  * @returns {Overlay}
  */
 function PopUp(action, options, dialogueid, trigger_elmnt) {
 	var overlay = overlays_stack.getById(dialogueid);
 	if (!overlay) {
-		var wide_popup_actions = ['popup.generic', 'popup.scriptexec', 'dashboard.share.edit',
-				'dashboard.properties.edit', 'popup.services', 'popup.media', 'popup.lldoperation', 'popup.lldoverride',
-				'popup.preproctest.edit', 'popup.triggerexpr', 'popup.httpstep', 'popup.testtriggerexpr',
-				'popup.triggerwizard'
+		var wide_popup_actions = ['popup.generic', 'dashboard.share.edit', 'dashboard.page.properties.edit',
+			'dashboard.properties.edit', 'dashboard.widget.edit', 'popup.services', 'popup.media', 'popup.lldoperation',
+				'popup.lldoverride', 'popup.preproctest.edit', 'popup.triggerexpr', 'popup.httpstep',
+				'popup.testtriggerexpr', 'popup.triggerwizard'
 			],
-			medium_popup_actions = ['popup.maintenance.period', 'popup.condition.actions', 'popup.action.recovery',
-				'popup.action.acknowledge', 'popup.action.operation', 'popup.condition.operations',
+			medium_popup_actions = ['popup.maintenance.period', 'popup.condition.actions', 'popup.condition.operations',
 				'popup.condition.event.corr', 'popup.discovery.check', 'popup.mediatypetest.edit',
-				'popup.mediatype.message'
+				'popup.mediatype.message', 'popup.scriptexec', 'popup.scheduledreport.test'
 			],
+			static_popup_actions = ['popup.massupdate.template', 'popup.massupdate.host', 'popup.massupdate.trigger',
+				'popup.massupdate.triggerprototype'
+			],
+			preprocessing_popup_actions = ['popup.massupdate.item', 'popup.massupdate.itemprototype'],
 			dialogue_class = '';
 
 		if (wide_popup_actions.indexOf(action) !== -1) {
@@ -339,6 +357,12 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 		else if (medium_popup_actions.indexOf(action) !== -1) {
 			dialogue_class = ' modal-popup-medium';
 		}
+		else if (static_popup_actions.indexOf(action) !== -1) {
+			dialogue_class = ' modal-popup-static';
+		}
+		else if (preprocessing_popup_actions.indexOf(action) !== -1) {
+			dialogue_class = ' modal-popup-preprocessing';
+		}
 
 		overlay = overlayDialogue({
 			'dialogueid': dialogueid,
@@ -362,12 +386,25 @@ function PopUp(action, options, dialogueid, trigger_elmnt) {
 			else {
 				var buttons = resp.buttons !== null ? resp.buttons : [];
 
-				buttons.push({
-					'title': t('Cancel'),
-					'class': 'btn-alt',
-					'cancel': true,
-					'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
-				});
+				switch (action) {
+					case 'popup.scheduledreport.list':
+					case 'popup.scheduledreport.test':
+					case 'popup.scriptexec':
+						buttons.push({
+							'title': t('Ok'),
+							'cancel': true,
+							'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
+						});
+						break;
+
+					default:
+						buttons.push({
+							'title': t('Cancel'),
+							'class': 'btn-alt',
+							'cancel': true,
+							'action': (typeof resp.cancel_action !== 'undefined') ? resp.cancel_action : function() {}
+						});
+				}
 
 				overlay.setProperties({
 					title: resp.header,
@@ -455,11 +492,6 @@ function addToOverlaysStack(id, element, type, xhr) {
 // Keydown handler. Closes last opened overlay UI element.
 function closeDialogHandler(event) {
 	if (event.which == 27) { // ESC
-		// Do not catch multiselect events.
-		if (jQuery(event.target).closest('.multiselect').data('multiSelect') !== undefined) {
-			return;
-		}
-
 		var dialog = overlays_stack.end();
 		if (typeof dialog !== 'undefined') {
 			switch (dialog.type) {
@@ -656,7 +688,7 @@ function addSelectedValues(object, parentid) {
  */
 function add_media(formname, media, mediatypeid, sendto, period, active, severity) {
 	var form = window.document.forms[formname];
-	var media_name = (media > -1) ? 'user_medias[' + media + ']' : 'new_media';
+	var media_name = (media > -1) ? 'medias[' + media + ']' : 'new_media';
 
 	window.create_var(form, media_name + '[mediatypeid]', mediatypeid);
 	if (typeof sendto === "object") {
@@ -939,29 +971,29 @@ Function.prototype.bindAsEventListener = function (context) {
 	};
 };
 
-/**
- * Get first selected host from multiselect field.
- *
- * @param {string} host_field_id       Host field element ID.
- * @param {string} hostgroup_field_id  Host group field element ID.
- *
- * @return {object}
- */
-function getFirstMultiselectValue(host_field_id, hostgroup_field_id) {
-	var host_values = (typeof host_field_id !== 'undefined')
-			? jQuery('#'+host_field_id).multiSelect('getData')
-			: [],
-		hostgroup_values = (typeof hostgroup_field_id !== 'undefined')
-			? jQuery('#'+hostgroup_field_id).multiSelect('getData')
-			: [],
-		ret = {};
+function openMassupdatePopup(elem, popup_name) {
+	const data = {};
+	const form = elem.closest('form');
 
-	if (host_values.length != 0) {
-		ret.hostid = host_values[0].id;
-	}
-	if (hostgroup_values.length != 0) {
-		ret.groupid = hostgroup_values[0].id;
+	data.ids = chkbxRange.selectedIds;
+
+	switch (popup_name) {
+		case 'popup.massupdate.item':
+			data['context'] = form.querySelector('#context').value;
+			data['prototype'] = 0;
+			break;
+
+		case 'popup.massupdate.trigger':
+			data['context'] = form.querySelector('#context').value;
+			break;
+
+		case 'popup.massupdate.itemprototype':
+		case 'popup.massupdate.triggerprototype':
+			data['parent_discoveryid'] = form.querySelector('#parent_discoveryid').value;
+			data['context'] = form.querySelector('#context').value;
+			data['prototype'] = 1;
+			break;
 	}
 
-	return ret;
+	return PopUp(popup_name, data, null, this);
 }
