@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -896,7 +896,7 @@ class CWidgetNavTree extends CWidget {
 												jQuery('> .tree-row > .content > .item-name', $row)
 													.empty()
 													.attr('title', resp['name'])
-													.append(jQuery('<span/>').text(resp.name));
+													.append(jQuery('<span>').text(resp.name));
 												$row.toggleClass('no-map', resp.sysmapid == 0);
 											}
 											else {
