@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of triggers');
 $page['file'] = 'triggers.php';
-$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'class.tab-indicators.js'];
+$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'class.tab-indicators.js', 'class.tagfilteritem.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -66,10 +66,6 @@ $fields = [
 	'copy_targetids' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,			null],
 	'visible' =>								[T_ZBX_STR, O_OPT, null,	null,			null],
 	'tags' =>									[T_ZBX_STR, O_OPT, null,	null,			null],
-	'mass_update_tags'	=>						[T_ZBX_INT, O_OPT, null,
-													IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]),
-													null
-												],
 	'show_inherited_tags' =>					[T_ZBX_INT, O_OPT, null,	IN([0,1]),		null],
 	'manual_close' =>							[T_ZBX_INT, O_OPT, null,
 													IN([ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED,
@@ -77,6 +73,7 @@ $fields = [
 													]),
 													null
 												],
+	'context' =>								[T_ZBX_STR, O_MAND, P_SYS,	IN('"host", "template"'),	null],
 	// Filter related fields.
 	'filter_set' =>								[T_ZBX_STR, O_OPT, P_SYS,	null,			null],
 	'filter_rst' =>								[T_ZBX_STR, O_OPT, P_SYS,	null,			null],
@@ -110,7 +107,7 @@ $fields = [
 	// Action related fields.
 	'action' =>									[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 													IN('"trigger.masscopyto","trigger.massdelete","trigger.massdisable",'.
-														'"trigger.massenable","trigger.massupdate","trigger.massupdateform"'
+														'"trigger.massenable"'
 													),
 													null
 												],
@@ -135,7 +132,6 @@ $fields = [
 	'clone' =>									[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'add' =>									[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'update' =>									[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
-	'massupdate' =>								[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'delete' =>									[T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null],
 	'cancel' =>									[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form' =>									[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
@@ -188,15 +184,13 @@ if (getRequest('hostid') && !isWritableHostTemplates([getRequest('hostid')])) {
 }
 
 $tags = getRequest('tags', []);
+
+// Unset empty and inherited tags.
 foreach ($tags as $key => $tag) {
-	// remove empty new tag lines
 	if ($tag['tag'] === '' && $tag['value'] === '') {
 		unset($tags[$key]);
-		continue;
 	}
-
-	// remove inherited tags
-	if (array_key_exists('type', $tag) && !($tag['type'] & ZBX_PROPERTY_OWN)) {
+	elseif (array_key_exists('type', $tag) && !($tag['type'] & ZBX_PROPERTY_OWN)) {
 		unset($tags[$key]);
 	}
 	else {
@@ -432,100 +426,6 @@ elseif (isset($_REQUEST['add_dependency']) && isset($_REQUEST['new_dependency'])
 		}
 	}
 }
-elseif (hasRequest('action') && getRequest('action') === 'trigger.massupdate'
-		&& hasRequest('massupdate') && hasRequest('g_triggerid')) {
-	$result = true;
-	$visible = getRequest('visible', []);
-
-	if ($visible) {
-		$triggerids = getRequest('g_triggerid');
-		$triggers_to_update = [];
-
-		$options = [
-			'output' => ['triggerid', 'templateid'],
-			'triggerids' => $triggerids,
-			'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL],
-			'preservekeys' => true
-		];
-
-		if (array_key_exists('tags', $visible)) {
-			$mass_update_tags = getRequest('mass_update_tags', ZBX_ACTION_ADD);
-
-			if ($mass_update_tags == ZBX_ACTION_ADD || $mass_update_tags == ZBX_ACTION_REMOVE) {
-				$options['selectTags'] = ['tag', 'value'];
-			}
-
-			$unique_tags = [];
-
-			foreach ($tags as $tag) {
-				$unique_tags[$tag['tag'].':'.$tag['value']] = $tag;
-			}
-
-			$tags = array_values($unique_tags);
-		}
-
-		$triggers = API::Trigger()->get($options);
-
-		if ($triggers) {
-			foreach ($triggerids as $triggerid) {
-				if (array_key_exists($triggerid, $triggers)) {
-					$trigger = ['triggerid' => $triggerid];
-
-					if (array_key_exists('priority', $visible)) {
-						$trigger['priority'] = getRequest('priority');
-					}
-					if (array_key_exists('dependencies', $visible)) {
-						$trigger['dependencies'] = zbx_toObject(getRequest('dependencies', []), 'triggerid');
-					}
-					if (array_key_exists('tags', $visible)) {
-						if ($tags && $mass_update_tags == ZBX_ACTION_ADD) {
-							$unique_tags = [];
-
-							foreach (array_merge($triggers[$triggerid]['tags'], $tags) as $tag) {
-								$unique_tags[$tag['tag'].':'.$tag['value']] = $tag;
-							}
-
-							$trigger['tags'] = array_values($unique_tags);
-						}
-						elseif ($mass_update_tags == ZBX_ACTION_REPLACE) {
-							$trigger['tags'] = $tags;
-						}
-						elseif ($tags && $mass_update_tags == ZBX_ACTION_REMOVE) {
-							$diff_tags = [];
-
-							foreach ($triggers[$triggerid]['tags'] as $a) {
-								foreach ($tags as $b) {
-									if ($a['tag'] === $b['tag'] && $a['value'] === $b['value']) {
-										continue 2;
-									}
-								}
-
-								$diff_tags[] = $a;
-							}
-
-							$trigger['tags'] = $diff_tags;
-						}
-					}
-					if ($triggers[$triggerid]['templateid'] == 0 && array_key_exists('manual_close', $visible)) {
-						$trigger['manual_close'] = getRequest('manual_close');
-					}
-
-					$triggers_to_update[] = $trigger;
-				}
-			}
-		}
-
-		if ($triggers_to_update) {
-			$result = (bool) API::Trigger()->update($triggers_to_update);
-		}
-	}
-
-	if ($result) {
-		unset($_REQUEST['form'], $_REQUEST['g_triggerid']);
-		uncheckTableRows(getRequest('checkbox_hash'));
-	}
-	show_messages($result, _('Trigger updated'), _('Cannot update trigger'));
-}
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['trigger.massenable', 'trigger.massdisable']) && hasRequest('g_triggerid')) {
 	$enable = (getRequest('action') === 'trigger.massenable');
 	$status = $enable ? TRIGGER_STATUS_ENABLED : TRIGGER_STATUS_DISABLED;
@@ -626,14 +526,7 @@ elseif (hasRequest('action') && getRequest('action') === 'trigger.massdelete' &&
 /*
  * Display
  */
-if ((getRequest('action') === 'trigger.massupdateform' || hasRequest('massupdate')) && hasRequest('g_triggerid')) {
-	$data = getTriggerMassupdateFormData();
-	$data['action'] = 'trigger.massupdate';
-
-	// render view
-	echo (new CView('configuration.triggers.massupdate', $data))->getOutput();
-}
-elseif (isset($_REQUEST['form'])) {
+if (isset($_REQUEST['form'])) {
 	$data = [
 		'form' => getRequest('form'),
 		'form_refresh' => getRequest('form_refresh'),
@@ -666,7 +559,8 @@ elseif (isset($_REQUEST['form'])) {
 		'show_inherited_tags' => getRequest('show_inherited_tags', 0),
 		'correlation_mode' => getRequest('correlation_mode', ZBX_TRIGGER_CORRELATION_NONE),
 		'correlation_tag' => getRequest('correlation_tag', ''),
-		'manual_close' => getRequest('manual_close', ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED)
+		'manual_close' => getRequest('manual_close', ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED),
+		'context' => getRequest('context')
 	];
 
 	// render view
@@ -680,6 +574,12 @@ elseif (hasRequest('action') && getRequest('action') === 'trigger.masscopyto' &&
 	echo (new CView('configuration.copy.elements', $data))->getOutput();
 }
 else {
+	$data = [
+		'context' => getRequest('context')
+	];
+
+	$prefix = ($data['context'] === 'host') ? 'web.hosts.' : 'web.templates.';
+
 	$filter_groupids_ms = [];
 	$filter_hostids_ms = [];
 
@@ -704,7 +604,7 @@ else {
 		$filter_name = '';
 		$filter_priority = [];
 		$filter_groupids = [];
-		$filter_hostids = getRequest('filter_hostids', CProfile::getArray('web.triggers.filter_hostids', []));
+		$filter_hostids = getRequest('filter_hostids', CProfile::getArray($prefix.'triggers.filter_hostids', []));
 		if (count($filter_hostids) != 1) {
 			$filter_hostids = [];
 		}
@@ -715,25 +615,25 @@ else {
 		$filter_tags = [];
 	}
 	else {
-		$filter_inherited = CProfile::get('web.triggers.filter_inherited', -1);
-		$filter_discovered = CProfile::get('web.triggers.filter_discovered', -1);
-		$filter_dependent = CProfile::get('web.triggers.filter_dependent', -1);
-		$filter_name = CProfile::get('web.triggers.filter_name', '');
-		$filter_priority = CProfile::getArray('web.triggers.filter_priority', []);
-		$filter_groupids = CProfile::getArray('web.triggers.filter_groupids', []);
-		$filter_hostids = CProfile::getArray('web.triggers.filter_hostids', []);
-		$filter_state = CProfile::get('web.triggers.filter_state', -1);
-		$filter_status = CProfile::get('web.triggers.filter_status', -1);
-		$filter_value = CProfile::get('web.triggers.filter_value', -1);
-		$filter_evaltype = CProfile::get('web.triggers.filter.evaltype', TAG_EVAL_TYPE_AND_OR);
+		$filter_inherited = CProfile::get($prefix.'triggers.filter_inherited', -1);
+		$filter_discovered = CProfile::get($prefix.'triggers.filter_discovered', -1);
+		$filter_dependent = CProfile::get($prefix.'triggers.filter_dependent', -1);
+		$filter_name = CProfile::get($prefix.'triggers.filter_name', '');
+		$filter_priority = CProfile::getArray($prefix.'triggers.filter_priority', []);
+		$filter_groupids = CProfile::getArray($prefix.'triggers.filter_groupids', []);
+		$filter_hostids = CProfile::getArray($prefix.'triggers.filter_hostids', []);
+		$filter_state = CProfile::get($prefix.'triggers.filter_state', -1);
+		$filter_status = CProfile::get($prefix.'triggers.filter_status', -1);
+		$filter_value = CProfile::get($prefix.'triggers.filter_value', -1);
+		$filter_evaltype = CProfile::get($prefix.'triggers.filter.evaltype', TAG_EVAL_TYPE_AND_OR);
 
 		$filter_tags = [];
 
-		foreach (CProfile::getArray('web.triggers.filter.tags.tag', []) as $i => $tag) {
+		foreach (CProfile::getArray($prefix.'triggers.filter.tags.tag', []) as $i => $tag) {
 			$filter_tags[] = [
 				'tag' => $tag,
-				'value' => CProfile::get('web.triggers.filter.tags.value', null, $i),
-				'operator' => CProfile::get('web.triggers.filter.tags.operator', null, $i)
+				'value' => CProfile::get($prefix.'triggers.filter.tags.value', null, $i),
+				'operator' => CProfile::get($prefix.'triggers.filter.tags.operator', null, $i)
 			];
 		}
 	}
@@ -752,14 +652,27 @@ else {
 	}
 
 	if ($filter_hostids) {
-		$filter_hostids = API::Host()->get([
-			'output' => ['hostid', 'name'],
-			'hostids' => $filter_hostids,
-			'templated_hosts' => true,
-			'editable' => true,
-			'preservekeys' => true
-		]);
-		$filter_hostids_ms = CArrayHelper::renameObjectsKeys($filter_hostids, ['hostid' => 'id']);
+		if ($data['context'] === 'host') {
+			$filter_hostids = API::Host()->get([
+				'output' => ['hostid', 'name'],
+				'hostids' => $filter_hostids,
+				'editable' => true,
+				'preservekeys' => true
+			]);
+
+			$filter_hostids_ms = CArrayHelper::renameObjectsKeys($filter_hostids, ['hostid' => 'id']);
+		}
+		else {
+			$filter_hostids = API::Template()->get([
+				'output' => ['templateid', 'name'],
+				'templateids' => $filter_hostids,
+				'editable' => true,
+				'preservekeys' => true
+			]);
+
+			$filter_hostids_ms = CArrayHelper::renameObjectsKeys($filter_hostids, ['templateid' => 'id']);
+		}
+
 		$filter_hostids = array_keys($filter_hostids_ms);
 	}
 
@@ -768,9 +681,9 @@ else {
 		return (bool) $v['tag'];
 	});
 
-	$sort = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'description'));
-	$sortorder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
-	$active_tab = CProfile::get('web.triggers.filter.active', 1);
+	$sort = getRequest('sort', CProfile::get($prefix.$page['file'].'.sort', 'description'));
+	$sortorder = getRequest('sortorder', CProfile::get($prefix.$page['file'].'.sortorder', ZBX_SORT_UP));
+	$active_tab = CProfile::get($prefix.'triggers.filter.active', 1);
 
 	// Get triggers (build options).
 	$options = [
@@ -779,7 +692,7 @@ else {
 		'groupids' => $filter_groupids ? $filter_groupids_enriched : null,
 		'editable' => true,
 		'dependent' => ($filter_dependent != -1) ? $filter_dependent : null,
-		'templated' => ($filter_value != -1) ? false : null,
+		'templated' => ($filter_value == -1) ? ($data['context'] === 'template') : false,
 		'inherited' => ($filter_inherited != -1) ? $filter_inherited : null,
 		'preservekeys' => true,
 		'sortfield' => $sort,
@@ -850,7 +763,9 @@ else {
 
 	CPagerHelper::savePage($page['file'], $page_num);
 
-	$paging = CPagerHelper::paginate($page_num, $prefetched_triggers, $sortorder, new CUrl('triggers.php'));
+	$paging = CPagerHelper::paginate($page_num, $prefetched_triggers, $sortorder,
+		(new CUrl('triggers.php'))->setArgument('context', $data['context'])
+	);
 
 	// fetch triggers
 	$triggers = [];
@@ -910,12 +825,15 @@ else {
 
 	$show_info_column = false;
 	$show_value_column = false;
-	foreach ($triggers as $trigger) {
-		foreach ($trigger['hosts'] as $trigger_host) {
-			if (in_array($trigger_host['status'], [HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED])) {
-				$show_value_column = true;
-				$show_info_column = true;
-				break 2;
+
+	if ($data['context'] === 'host') {
+		foreach ($triggers as $trigger) {
+			foreach ($trigger['hosts'] as $trigger_host) {
+				if (in_array($trigger_host['status'], [HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED])) {
+					$show_value_column = true;
+					$show_info_column = true;
+					break 2;
+				}
 			}
 		}
 	}
@@ -948,20 +866,20 @@ else {
 		unset($dependencyTrigger);
 	}
 
-	CProfile::update('web.'.$page['file'].'.sort', $sort, PROFILE_TYPE_STR);
-	CProfile::update('web.'.$page['file'].'.sortorder', $sortorder, PROFILE_TYPE_STR);
+	CProfile::update($prefix.$page['file'].'.sort', $sort, PROFILE_TYPE_STR);
+	CProfile::update($prefix.$page['file'].'.sortorder', $sortorder, PROFILE_TYPE_STR);
 
 	if (getRequest('filter_set')) {
-		CProfile::update('web.triggers.filter_inherited', $filter_inherited, PROFILE_TYPE_INT);
-		CProfile::update('web.triggers.filter_discovered', $filter_discovered, PROFILE_TYPE_INT);
-		CProfile::update('web.triggers.filter_dependent', $filter_dependent, PROFILE_TYPE_INT);
-		CProfile::update('web.triggers.filter_name', $filter_name, PROFILE_TYPE_STR);
-		CProfile::updateArray('web.triggers.filter_priority', $filter_priority, PROFILE_TYPE_INT);
-		CProfile::updateArray('web.triggers.filter_groupids', $filter_groupids, PROFILE_TYPE_ID);
-		CProfile::updateArray('web.triggers.filter_hostids', $filter_hostids, PROFILE_TYPE_ID);
-		CProfile::update('web.triggers.filter_state', $filter_state, PROFILE_TYPE_INT);
-		CProfile::update('web.triggers.filter_status', $filter_status, PROFILE_TYPE_INT);
-		CProfile::update('web.triggers.filter.evaltype', $filter_evaltype, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter_inherited', $filter_inherited, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter_discovered', $filter_discovered, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter_dependent', $filter_dependent, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter_name', $filter_name, PROFILE_TYPE_STR);
+		CProfile::updateArray($prefix.'triggers.filter_priority', $filter_priority, PROFILE_TYPE_INT);
+		CProfile::updateArray($prefix.'triggers.filter_groupids', $filter_groupids, PROFILE_TYPE_ID);
+		CProfile::updateArray($prefix.'triggers.filter_hostids', $filter_hostids, PROFILE_TYPE_ID);
+		CProfile::update($prefix.'triggers.filter_state', $filter_state, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter_status', $filter_status, PROFILE_TYPE_INT);
+		CProfile::update($prefix.'triggers.filter.evaltype', $filter_evaltype, PROFILE_TYPE_INT);
 
 		$filter_tags_fmt = ['tags' => [], 'values' => [], 'operators' => []];
 
@@ -975,35 +893,35 @@ else {
 			$filter_tags_fmt['operators'][] = $filter_tag['operator'];
 		}
 
-		CProfile::updateArray('web.triggers.filter.tags.tag', $filter_tags_fmt['tags'], PROFILE_TYPE_STR);
-		CProfile::updateArray('web.triggers.filter.tags.value', $filter_tags_fmt['values'], PROFILE_TYPE_STR);
-		CProfile::updateArray('web.triggers.filter.tags.operator', $filter_tags_fmt['operators'], PROFILE_TYPE_INT);
+		CProfile::updateArray($prefix.'triggers.filter.tags.tag', $filter_tags_fmt['tags'], PROFILE_TYPE_STR);
+		CProfile::updateArray($prefix.'triggers.filter.tags.value', $filter_tags_fmt['values'], PROFILE_TYPE_STR);
+		CProfile::updateArray($prefix.'triggers.filter.tags.operator', $filter_tags_fmt['operators'], PROFILE_TYPE_INT);
 
 		if ($show_value_column) {
-			CProfile::update('web.triggers.filter_value', $filter_value, PROFILE_TYPE_INT);
+			CProfile::update($prefix.'triggers.filter_value', $filter_value, PROFILE_TYPE_INT);
 		}
 	}
 	elseif (getRequest('filter_rst')) {
-		CProfile::deleteIdx('web.triggers.filter_inherited');
-		CProfile::deleteIdx('web.triggers.filter_discovered');
-		CProfile::deleteIdx('web.triggers.filter_dependent');
-		CProfile::deleteIdx('web.triggers.filter_name');
-		CProfile::deleteIdx('web.triggers.filter_priority');
-		CProfile::deleteIdx('web.triggers.filter_groupids');
+		CProfile::deleteIdx($prefix.'triggers.filter_inherited');
+		CProfile::deleteIdx($prefix.'triggers.filter_discovered');
+		CProfile::deleteIdx($prefix.'triggers.filter_dependent');
+		CProfile::deleteIdx($prefix.'triggers.filter_name');
+		CProfile::deleteIdx($prefix.'triggers.filter_priority');
+		CProfile::deleteIdx($prefix.'triggers.filter_groupids');
 
 		if (count($filter_hostids) != 1) {
-			CProfile::deleteIdx('web.triggers.filter_hostids');
+			CProfile::deleteIdx($prefix.'triggers.filter_hostids');
 		}
 
-		CProfile::deleteIdx('web.triggers.filter_state');
-		CProfile::deleteIdx('web.triggers.filter_status');
-		CProfile::deleteIdx('web.triggers.filter.evaltype');
-		CProfile::deleteIdx('web.triggers.filter.tags.tag');
-		CProfile::deleteIdx('web.triggers.filter.tags.value');
-		CProfile::deleteIdx('web.triggers.filter.tags.operator');
+		CProfile::deleteIdx($prefix.'triggers.filter_state');
+		CProfile::deleteIdx($prefix.'triggers.filter_status');
+		CProfile::deleteIdx($prefix.'triggers.filter.evaltype');
+		CProfile::deleteIdx($prefix.'triggers.filter.tags.tag');
+		CProfile::deleteIdx($prefix.'triggers.filter.tags.value');
+		CProfile::deleteIdx($prefix.'triggers.filter.tags.operator');
 
 		if ($show_value_column) {
-			CProfile::deleteIdx('web.triggers.filter_value');
+			CProfile::deleteIdx($prefix.'triggers.filter_value');
 		}
 	}
 
@@ -1015,9 +933,9 @@ else {
 	sort($filter_hostids);
 	$checkbox_hash = crc32(implode('', $filter_hostids));
 
-	$data = [
+	$data += [
 		'triggers' => $triggers,
-		'profileIdx' => 'web.triggers.filter',
+		'profileIdx' => $prefix.'triggers.filter',
 		'active_tab' => $active_tab,
 		'sort' => $sort,
 		'sortorder' => $sortorder,
